/*
 * admin namespace support
 * 
 * Copyright (C) 2003, 2004 Lars Lindner <lars.lindner@gmx.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>
#include "common.h"
#include "ns_admin.h"

/* you can find an admin namespace spec at:
 * http://web.resource.org/rss/1.0/modules/admin/
 * 
 * taglist for admin:
 * --------------------------------
 * errorReportsTo
 * generatorAgent
 * --------------------------------
 * 
 * both tags usually contains URIs which we simply display in the
 * feed info view footer
 */

/************************************************************************/
/*                        PRIVATE FUNCTIONS                             */
/************************************************************************/

static void parse_channel_tag(feedPtr fp, xmlNodePtr cur)
{
    gchar *value = NULL;

    value = utf8_fix(xmlGetProp(cur, (const xmlChar *)"resource"));

    if (!xmlStrcmp((const xmlChar *)"errorReportsTo", cur->name))
        metadata_list_set(&(fp->metadata), "errorReportsTo", value);
    else if (!xmlStrcmp((const xmlChar *)"generatorAgent", cur->name))
        metadata_list_set(&(fp->metadata), "feedgeneratorUri", value);

    g_free(value);
    return;
}

static void ns_admin_register_ns(NsHandler * nsh, GHashTable * prefixhash,
                                 GHashTable * urihash)
{
    g_hash_table_insert(prefixhash, "admin", nsh);
    g_hash_table_insert(urihash, "http://webns.net/mvcb/", nsh);
}

/************************************************************************/
/*                        PUBLIC FUNCTIONS                              */
/************************************************************************/

NsHandler *ns_admin_getRSSNsHandler(void)
{
    NsHandler *nsh = NULL;

    nsh = g_new0(NsHandler, 1);
    nsh->registerNs = ns_admin_register_ns;
    nsh->prefix = "admin";
    nsh->parseChannelTag = parse_channel_tag;;

    return nsh;
}
